# -*- coding: utf-8 -*-
#
# Marsu Plugin by gutemine
#
marsu_version="1.1"
#
from Components.ActionMap import ActionMap
from Components.Label import Label
from Components.config import config, ConfigSubsection, ConfigText, ConfigBoolean, ConfigInteger, ConfigSelection, getConfigListEntry
from Components.ConfigList import ConfigListScreen
from Plugins.Plugin import PluginDescriptor
from Components.Pixmap import Pixmap
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox 
from Screens.InputBox import InputBox
from Components.Input import Input
from Screens.ChoiceBox import ChoiceBox
from Components.AVSwitch import AVSwitch
from Components.SystemInfo import SystemInfo
from Screens.Console import Console                                                                           
from Components.MenuList import MenuList       
from enigma import  quitMainloop, eConsoleAppContainer, eDVBVolumecontrol, eTimer
from Tools.LoadPixmap import LoadPixmap
import Screens.Standby  
import sys, os, struct, stat, time
from fcntl import ioctl
from struct import unpack
from array import array

from os import statvfs, path as os_path, chmod as os_chmod, write as os_write, open as os_open,\
		close as os_close, unlink as os_unlink
from twisted.web import resource, http
import gettext, datetime

marsu_plugindir="/usr/lib/enigma2/python/Plugins/Extensions/Marsu" 
marsu_pluginlink="/tmp/Marsu" 
marsu_bin="%s/bin" % marsu_pluginlink
marsu_busy="/tmp/.marsu"
marsu_done="/tmp/.marsudone"
marsu_script="/tmp/marsu.sh"
marsu_log="/tmp/marsu.log"

if not os.path.exists(marsu_pluginlink):
	os.symlink(marsu_plugindir,marsu_pluginlink)
# add local language file
marsu_sp=config.osd.language.value.split("_")
marsu_language = marsu_sp[0]
if os.path.exists("%s/locale/%s" % (marsu_plugindir,marsu_language)):
	_=gettext.Catalog('marsu', '%s/locale' % marsu_plugindir,marsu_sp).gettext

boxtype="dm7020hd"
if os.path.exists("/proc/stb/info/model"):
	f=open("/proc/stb/info/model")
	boxtype=f.read()
	f.close()
	boxtype=boxtype.replace("\n","").replace("\l","")

yes_no_descriptions = {False: _("no"), True: _("yes")}    

config.plugins.marsu = ConfigSubsection()
config.plugins.marsu.conversionlocation = ConfigText(default = "/media/hdd/backup", fixed_size=False, visible_width=40)
config.plugins.marsu.swapsize = ConfigInteger(default = 256, limits = (0, 2047))
if boxtype == "dm8000":
	config.plugins.marsu.volsize = ConfigInteger(default = 248, limits = (60, 248))
elif boxtype.startswith("dm7020hd"):
	config.plugins.marsu.volsize = ConfigInteger(default = 397, limits = (60, 950))
else:
	config.plugins.marsu.volsize = ConfigInteger(default = 60, limits = (40, 60))
config.plugins.marsu.summary = ConfigBoolean(default = True, descriptions=yes_no_descriptions)
config.plugins.marsu.console = ConfigBoolean(default = False, descriptions=yes_no_descriptions)
if boxtype != "dm8000" and boxtype != "dm7020hd":                                
	config.plugins.marsu.summary.value = False

jcompression=[]                    
#jcompression.append(( "lzo", _("lzo") ))
jcompression.append(( "zlib", _("zlib") ))
jcompression.append(( "none", _("none") ))
if boxtype == "dm8000" or boxtype == "dm7020hd":
	config.plugins.marsu.jffs2compression = ConfigSelection(default = "zlib", choices = jcompression)
#	config.plugins.marsu.jffs2compression = ConfigSelection(default = "none", choices = jcompression)
else:
	config.plugins.marsu.jffs2compression = ConfigSelection(default = "zlib", choices = jcompression)

ucompression=[]                    
ucompression.append(( "lzo", _("lzo") ))
ucompression.append(( "favor_lzo", _("favor_lzo") ))
ucompression.append(( "zlib", _("zlib") ))
if boxtype == "dm8000" or boxtype == "dm7020hd":
        ucompression.append(( "none", _("none") ))
config.plugins.marsu.ubifsrootcompression = ConfigSelection(default = "favor_lzo", choices = ucompression)
config.plugins.marsu.ubifsdatacompression = ConfigSelection(default = "favor_lzo", choices = ucompression)
config.plugins.marsu.databackup = ConfigBoolean(default = False, descriptions=yes_no_descriptions)

disclaimer_string=_("Disclaimer")
disclaimer_text=_("This way of creating an UBI image for your Dreambox is\npotentially dangerous and NOT supported in any way from DMM.\n\nYou are using it completely at you own risk!")
title_string=_("Marsu Plugin")
about_string=_("Marsu Plugin by gutemine Version %s") % marsu_version
setup_string=_("Configuring")
conversion_string=_("Conversion")
backup_string=_("Backup")
running_string=_("Marsu is busy ...")
convertimage_string=_("Enter Imagename")
convertimage_directory=_("Conversion Imagedirectory")
boxtype_string=_("Target")
model_string=_("Model")
unsupported_string=_("Sorry, currently not supported on this Dreambox")
nothing_selected_string=_("Nothing selected")
nothing_entered_string=_("Nothing entered")

class Marsu(Screen):
	skin = """
		<screen position="center,80" size="680,60" title="Flashing" >
		<widget name="logo" position="10,10" size="100,40" transparent="1" alphatest="on" />
		<widget name="buttonred" position="120,10" size="130,40" backgroundColor="red" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
		<widget name="buttongreen" position="260,10" size="130,40" backgroundColor="green" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
		<widget name="buttonyellow" position="400,10" size="130,40" backgroundColor="yellow" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
		<widget name="buttonblue" position="540,10" size="130,40" backgroundColor="blue" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
	</screen>"""
	def __init__(self, session, args = 0):
		Screen.__init__(self, session)
		self.onShown.append(self.setWindowTitle)      
                self.onLayoutFinish.append(self.byLayoutEnd)
		self["logo"] = Pixmap()
		self["buttonred"] = Label(_("Cancel"))
		m=open("/proc/mounts")
		mounts=m.read()
		m.close()
		if mounts.find("ubi0:rootfs") is -1:
			self["buttongreen"] = Label(conversion_string)
		else:
			self["buttongreen"] = Label(backup_string)
	       	self["buttonyellow"] = Label(disclaimer_string)
		self["buttonblue"] = Label(setup_string)
		
		self["setupActions"] = ActionMap([ "ColorActions", "SetupActions" ],
			{
			"green": self.convert,
			"red": self.leaving,
			"blue": self.config,
			"yellow": self.disclaimer,
			"save": self.leaving,
			"cancel": self.leaving,
			})

	def setWindowTitle(self):
		if os.path.exists(marsu_busy):                                                                   
      			self["logo"].instance.setPixmapFromFile("%s/hubi.png" % marsu_plugindir)
		else:                                                                            
      			self["logo"].instance.setPixmapFromFile("%s/marsu.png" % marsu_plugindir)
		self.setTitle(title_string)

        def byLayoutEnd(self):
                self["logo"].instance.setPixmapFromFile("%s/marsu.png" % marsu_plugindir)

	def leaving(self):
	        if os.path.exists(marsu_busy):
			self.session.open(MessageBox, running_string, MessageBox.TYPE_ERROR)
		else:
			self.close()
			
	def cancel(self):
		self.close(False)
		
        def disclaimer(self):
	        self.session.open(MessageBox, disclaimer_text, MessageBox.TYPE_WARNING)

	def convert(self):
	        if os.path.exists(marsu_busy):
			self.session.open(MessageBox, running_string, MessageBox.TYPE_ERROR)
		else:
			if os.path.exists(marsu_done):
	 			print "[MARSU] found finished conversion ..."
				self.TimerConversion = eTimer()                                       
				self.TimerConversion.stop()                                           
				if os.path.exists(marsu_busy):
					os.remove(marsu_busy)
				if os.path.exists(marsu_script):
					os.remove(marsu_script)
				f=open(marsu_done)
				line=f.readline()
				f.close()
				os.remove(marsu_done)
				sp=[]
				sp=line.split("	")
				print sp
				length=len(sp)
				size=""
				image=""
				path=""
				if length > 0:
					size=sp[0].rstrip().lstrip()
					sp2=[]
					sp2=sp[length-1].split("/")
					print sp2
					length=len(sp2)
					if length > 0:
						image=sp2[length-1]
						path=line.replace(size,"").replace(image,"")
						image=image.replace(".nfi\n","")
						image=image.rstrip().lstrip()
				print "[MARSU] found conversion %s" % line
				self.session.open(MessageBox,size+"B "+_("Image Conversion to %s finished with imagename:\n\n%s.nfi\n\nYou may now Flash this image!") % (path,image),  MessageBox.TYPE_INFO)                 
			else:
				f=open("/proc/stb/info/model")
			        self.boxtype=f.read()
			        f.close() 
			        self.boxtype=self.boxtype.replace("\n","").replace("\l","")
  				if self.boxtype == "dm7025" or self.boxtype == "dm800pvr":
					self.session.open(MessageBox, unsupported_string, MessageBox.TYPE_ERROR)
				else:
	  				f=open("/proc/stb/info/model")
		  			self.boxtype=f.read()
  					f.close()
	  				self.boxtype=self.boxtype.replace("\n","").replace("\l","")
					name="OE"
					if os.path.exists("/etc/image-version"):
						f=open("/etc/image-version")
	 					line = f.readline()                                                    
	        				while (line):                                             
		        				line = f.readline()                                                 
        						if line.startswith("creator="):                                    
								name=line
        					f.close()                                                              
						name=name.replace("creator=","")
						sp=[]
						if len(name) > 0:
							sp=name.split(" ")
							if len(sp) > 0:
								name=sp[0]
								name=name.replace("\n","")
					self.creator=name.rstrip().lstrip()
					self.imagetype="exp"
					if name == "OoZooN" and os.path.exists("/etc/issue.net"):
						f=open("/etc/issue.net")
						i=f.read()
						f.close()
						if i.find("xperimental") is -1:
							self.imagetype="rel"
					name=name+"-"+self.imagetype
        		        	self.session.openWithCallback(self.askForTargetName,InputBox, title=convertimage_string, text="%s-%s-%s-%s                        " % (name,self.boxtype,datetime.date.today(),time.strftime("%H-%M")), maxSize=40, type=Input.TEXT)

        def askForTargetName(self,name):
		if name is None:
        		self.session.open(MessageBox,nothing_entered_string,  MessageBox.TYPE_ERROR)                 
           	else:
	   		self.targetname=name.replace(" ","").replace("[","").replace("]","").replace(">","").replace("<","").replace("|","").rstrip().lstrip()
      	      		if os.path.exists("%s/%s.nfi" % (config.plugins.marsu.conversionlocation.value,self.targetname)):
               			self.session.openWithCallback(self.confirmedConversion,MessageBox,"%s.nfi" % self.targetname +"\n"+_("already exists,")+" "+_("overwrite ?"), MessageBox.TYPE_YESNO)
	      		else:
				self.confirmedConversion(True)

        def confirmedConversion(self,answer):
	      if answer is True:
	      	 	if os.path.exists("%s/%s.nfi" % (config.plugins.marsu.conversionlocation.value,self.targetname)):
	      			os.remove("%s/%s.nfi" % (config.plugins.marsu.conversionlocation.value,self.targetname))
	      	 	if os.path.exists("%s/%s.nfo" % (config.plugins.marsu.conversionlocation.value,self.targetname)):
	      			os.remove("%s/%s.nfo" % (config.plugins.marsu.conversionlocation.value,self.targetname))
		      	# check if swapfile too small
		        fm=open("/proc/meminfo")                                                
       	         	line = fm.readline()                                                    
       	         	swapspace=0                                                                  
                 	while (line):                                             
           	    		line = fm.readline()                                                 
           	    		if line.startswith("SwapTotal:"):                                    
              				swapspace=int(line.replace("SwapTotal: ","").replace("kB",""))/1000             
                 	fm.close()                                                              
	         	print "[MARSU] swapspace: %i MB" % swapspace
	         	if swapspace < 256: 
		 		action_string=_("\n\nUsing temporary swapspace.")
				self.ownswap=False
			else:
		 		action_string=_("\n\nUsing existing swapspace.")
				self.ownswap=True
                 	self.session.openWithCallback(self.startConversion,MessageBox, _("Press OK for starting conversion to\n\n%s.nfi\n\nBe patient, this takes 3-5min ... ") % self.targetname + action_string, MessageBox.TYPE_INFO)
	      else:
              		self.session.open(MessageBox,_("not confirmed"),  MessageBox.TYPE_ERROR)                 
		
        def startConversion(self,answer):
              if answer is True:
	      	 print "[MARSU] is converting now ..."
                 self["logo"].instance.setPixmapFromFile("%s/hubi.png" % marsu_plugindir)
	         ConvertImage(self.session,self.targetname,self.ownswap)

	def config(self):
        	 self.session.open(MarsuConfiguration)

def main(session, **kwargs):
       	session.open(Marsu)   

def autostart(reason,**kwargs):
        if kwargs.has_key("session"):           
		session = kwargs["session"]                       
	print "[MARSU] autostart/stop"
	if os.path.exists(marsu_busy):
		os.remove(marsu_busy)
	m=open("/proc/mounts")
	mounts=m.read()
	m.close()
	if mounts.find("ubi0:rootfs") is not -1:
		f=open("/proc/stb/info/model")
		boxtype=f.read()
		f.close()
		boxtype=boxtype.replace("\n","").replace("\l","")
		print "[MARSU] found ubifs"
		# check autoexec.bat
		if mounts.find("/boot") is -1:
			os.system("mount -t jffs2 /dev/mtdblock2 /boot")
		else:
			os.system("mount -o remount,rw /boot")
		if os.path.exists("/boot/autoexec.bat"):
			a=open("/boot/autoexec.bat")
			autoexec=a.read()
			a.close()
			if autoexec.find("root=/dev/mtdblock3 rootfstype=jffs2") is not -1:
				print "[MARSU] patches autoexec.bat for ubifs"
				autoexecubi=autoexec.replace("root=/dev/mtdblock3 rootfstype=jffs2","ubi.mtd=root root=ubi0:rootfs rootfstype=ubifs")
				a=open("/boot/autoexec.bat","w")
				a.write(autoexecubi)
				a.close()
				os.system("mount -o remount,ro /boot")
		f=open("/etc/fstab")
		fstab=f.read()
		f.close()
		fstabadd=""
		if fstab.find("/dev/mtdblock2") is -1:
			print "[MARSU] patches fstab with /boot"
			fstabadd +="/dev/mtdblock2 /boot  jffs2 ro    0 0\n"
		if boxtype.startswith("dm7020hd") and fstab.find("/dev/ubi0_1") is -1:
			print "[MARSU] patches fstab with /data"
			fstabadd +="/dev/ubi0_1 /data  ubifs rw    0 0\n"
			if not os.path.exists("/data"):
				os.mkdir("/data")
		if len(fstabadd) > 1:
			f=open("/etc/fstab","a")
			f.write(fstabadd)
			f.close()
		if mounts.find("/boot") is -1:
			os.system("mount /boot")
		if boxtype.startswith("dm7020hd") and mounts.find("/data") is -1:
			os.system("mount /data")
		if mounts.find("/boot") is -1:
			os.system("mount /boot")
	
def sessionstart(reason, **kwargs):                                               
        if reason == 0 and "session" in kwargs:                                                        
		print "[MARSU] sessionstart"

def Plugins(**kwargs):
	return [PluginDescriptor(name="Marsu", description=_("UBI!"), where = PluginDescriptor.WHERE_PLUGINMENU, icon="marsu.png", fnc=main),
       		PluginDescriptor(where=PluginDescriptor.WHERE_SESSIONSTART, fnc=sessionstart, needsRestart=False),
       		PluginDescriptor(where=[PluginDescriptor.WHERE_SESSIONSTART, PluginDescriptor.WHERE_AUTOSTART], fnc=autostart)]

class ConvertImage(Screen):                                                      
        def __init__(self,session,targetname,ownswap):            
        	print "[MARSU] does conversion"
		os.system("touch %s" % marsu_busy)
        	self.session=session    
        	self.targetname=targetname               
        	self.ownswap=ownswap                
		f=open("/proc/stb/info/model")
	        self.boxtype=f.read()
	        f.close() 
	        self.boxtype=self.boxtype.replace("\n","").replace("\l","")
	        self.eraseblocksize=0
	        self.minimumiosize=0
	        self.lebsize=0
	        self.maxlebcountroot=0
	        self.maxlebcountdata=0
	        self.offset=0
	        self.rootsize=0
	        sp=[]
	        os.system("%s/mtdinfo /dev/mtd3 -u > /tmp/.mtdinfo 2>&1" % marsu_bin)
	        m=open("/tmp/.mtdinfo")
	        line=m.readline()
	        if line.find("error") is -1:
			# still jffs2 ...
		        while line:	
		        	if line.startswith("Eraseblock size:"):
			        	sp=line.split()
			        	self.eraseblocksize=int(sp[2])
		        	if line.startswith("Minimum input"):
			        	sp=line.split()
		        		self.minimumiosize=int(sp[4])
		        	if line.startswith("Default UBI LEB"):
			        	sp=line.split()
		        		self.lebsize=int(sp[4])
		        	if line.startswith("Default UBI VID header offset: "):
			        	sp=line.split()
		        		self.offset=int(sp[5])
		        	if line.startswith("Amount of eraseblocks:"):
		        		sp=line.split()
			        	self.rootsize=int(sp[4].replace("(","").replace(")",""))
			        line=m.readline()
			m.close()
		else:
			# already ubifs ...
			m.close()
		        os.system("%s/ubinfo -d 0 > /tmp/.mtdinfo 2>&1" % marsu_bin)              
		        m=open("/tmp/.mtdinfo")
		        line=m.readline()
	        	while line:	
                                if line.startswith("Logical eraseblock size:"):                 
                                        sp=line.split()                                         
	                                self.lebsize=int(sp[3])                                 
		        	if line.startswith("Minimum input"):
			        	sp=line.split()
			        	self.minimumiosize=int(sp[4])
		        	if line.startswith("Total amount of logical eraseblocks:"):
			        	sp=line.split()
			        	self.rootsize=int(sp[6].replace("(","").replace(")",""))
			        line=m.readline()
			m.close()
			# some parameters wie get from /dev/mtd2
	        	os.system("%s/mtdinfo /dev/mtd2 -u > /tmp/.mtdinfo 2>&1" % marsu_bin)
		        m=open("/tmp/.mtdinfo")
		        line=m.readline()
		        if line.find("error") is -1:
			        while line:	
			        	if line.startswith("Eraseblock size:"):
				        	sp=line.split()
				        	self.eraseblocksize=int(sp[2])
		        		if line.startswith("Default UBI VID header offset: "):
				        	sp=line.split()
			        		self.offset=int(sp[5])
				        line=m.readline()
			m.close()
			
		self.maxlebcountroot=config.plugins.marsu.volsize.value*1024*1024/self.lebsize
		self.maxlebcountdata=(971-config.plugins.marsu.volsize.value)*1024*1024/self.lebsize
		print "[MARSU] eraseblocksize %d bytes" % self.eraseblocksize
		print "[MARSU] minimumiosize %d bytes" % self.minimumiosize
		print "[MARSU] lebsize %d bytes" % self.lebsize
		print "[MARSU] rootsize %d bytes" % self.rootsize
		print "[MARSU] maxlebcount root %d bytes" % self.maxlebcountroot
		print "[MARSU] maxlebcount data %d bytes" % self.maxlebcountdata
		
		uc=open("/tmp/ubinize.cfg","w")
		c  ="[rootfs]\n"
		c +="mode=ubi\n"
		c +="image=%s/r.ubi\n" % config.plugins.marsu.conversionlocation.value
		c +="vol_id=0\n"
		c +="vol_name=rootfs\n"
		c +="vol_type=dynamic\n"
		if self.boxtype.startswith("dm7020hd"):
			v=int(config.plugins.marsu.volsize.value)
			c +="vol_size=%dMiB\n" % v
		else:
			c +="vol_flags=autoresize\n"
		if self.boxtype.startswith("dm7020hd"):
			c +="[data]\n"
			c +="mode=ubi\n"
			if config.plugins.marsu.databackup.value is True:  
				c +="image=%s/d.ubi\n" % config.plugins.marsu.conversionlocation.value
			c +="vol_id=1\n"
			c +="vol_name=data\n"
			c +="vol_type=dynamic\n"
			v=int(971-config.plugins.marsu.volsize.value)
			c +="vol_size=%dMiB\n" % v
			c +="vol_flags=autoresize\n"
		uc.write(c)
		uc.close()
		if self.boxtype.startswith("dm7020hd") and not os.path.exists("/data"):
			os.mkdir("/data")
			
		print "[MARSU] ubinize.cfg:\n%s\n" % c
		self.ubifsrootoptions="-m %d -e %d -c %d -F" % (self.minimumiosize,self.lebsize,self.maxlebcountroot)
		print "[MARSU] ubifs root options: %s" % self.ubifsrootoptions
		self.ubifsdataoptions="-m %d -e %d -c %d -F" % (self.minimumiosize,self.lebsize,self.maxlebcountdata)
		print "[MARSU] ubifs data options: %s" % self.ubifsdataoptions
		self.ubinizeoptions="-m %d -p %d -O %d" % (self.minimumiosize,self.eraseblocksize,self.offset)
		print "[MARSU] ubinize options: %s" % self.ubinizeoptions
        	
		if os.path.exists("%s/r.ubifs" % config.plugins.marsu.conversionlocation.value):
			os.remove("%s/r.ubifs" % config.plugins.marsu.conversionlocation.value)
		if os.path.exists("%s/r.ubi" % config.plugins.marsu.conversionlocation.value):
			os.remove("%s/r.ubi" % config.plugins.marsu.conversionlocation.value)
		if os.path.exists("%s/d.ubifs" % config.plugins.marsu.conversionlocation.value):
			os.remove("%s/d.ubifs" % config.plugins.marsu.conversionlocation.value)
		if os.path.exists("%s/d.ubi" % config.plugins.marsu.conversionlocation.value):
			os.remove("%s/d.ubi" % config.plugins.marsu.conversionlocation.value)
		if os.path.exists("%s/b.jffs2" % config.plugins.marsu.conversionlocation.value):
			os.remove("%s/b.jffs2" % config.plugins.marsu.conversionlocation.value)
		if os.path.exists("%s/s.bin" % config.plugins.marsu.conversionlocation.value):
			os.remove("%s/s.bin" % config.plugins.marsu.conversionlocation.value)
			
		# backup only as NFI2 if the loader is already NFI2 capable 
		if self.boxtype == "dm8000":
			self.brcmnand=""
		else:
			self.brcmnand="--brcmnand"
		self.flashsize="4000000"
		self.loadersize="40000"
		self.bootsize="3C0000"
		self.rootsize="3C00000"
#		self.blocksize="512"
		# better ?
	        self.blocksize=self.minimumiosize
		if self.boxtype.startswith("dm7020hd") or self.boxtype == "dm8000":
			self.flashsize="10000000"
			self.loadersize="100000"
			self.bootsize="700000"
			self.rootsize="F800000"
#			self.blocksize="2048"
		if self.boxtype.startswith("dm7020hd"):
#			self.blocksize="4096"
			self.rootsize="3F800000"

	        self.jffs2options=" -e 0x%x -n -l" % (self.eraseblocksize)
		print "[MARSU] jffs2 options: %s" % self.jffs2options
		self.buildoptions="%s/buildimage %s -a %s -e 0x%x -f 0x%s -s %s -b 0x%s:%s/s.bin -d 0x%s:%s/b.jffs2 -d 0x%s:%s/r.ubifs > %s/%s.nfi\n" % (marsu_bin,self.brcmnand,self.boxtype,self.eraseblocksize,self.flashsize,self.blocksize,self.loadersize,config.plugins.marsu.conversionlocation.value,self.bootsize,config.plugins.marsu.conversionlocation.value,self.rootsize,config.plugins.marsu.conversionlocation.value,config.plugins.marsu.conversionlocation.value,self.targetname)
		print "[MARSU] buildoptions: %s" % self.buildoptions
		self.creator="Marsu"
		mtdev="/dev/mtd1"     
		#	
		# here comes the fun ...
		#	
		command  ="#!/bin/sh -x\n"                                                                 
		command +="df -h\n"   
	      	if not self.ownswap and config.plugins.marsu.swapsize.value > 0: 
	      		# I do it my way ...
	      		swapsize=config.plugins.marsu.swapsize.value*1024
                	command +="swapoff %s/swapfile\n" % config.plugins.marsu.conversionlocation.value                        
                	command +="rm %s/swapfile\n" % config.plugins.marsu.conversionlocation.value                        
                	command +="dd if=/dev/zero of=%s/swapfile bs=1024 count=%d\n" % (config.plugins.marsu.conversionlocation.value,swapsize)
                	command +="mkswap %s/swapfile\n" % config.plugins.marsu.conversionlocation.value                                
                	command +="swapon %s/swapfile\n" % config.plugins.marsu.conversionlocation.value                                 
		command +="%s/nanddump -n -o -b -c -f %s/s.bin %s\n" % (marsu_bin,config.plugins.marsu.conversionlocation.value,mtdev)
		command +="umount /tmp/boot\n"
		command +="rm -r /tmp/boot\n"
		# get /boot from you own Dreambox
		if not os.path.exists("/boot/autoexec.bat"):
			os.system("mount -t jffs2 /dev/mtdblock2 /boot")
		command +="mkdir /tmp/boot\n"
		command +="cp /boot/* /tmp/boot; cd /tmp/boot; ln -sfn vmlinux-3.2-%s.gz vmlinux\n" % self.boxtype
		command +="ln -sfn /usr/share/bootlogo.mvi bootlogo.mvi; ln -sfn bootlog.mvi backdrop.mvi; ln -sfn bootlogo.mvi bootlogo_wait.mvi; cd /\n"
		# here comes autoexec for ubifs ...
		command +="sed -ie s!\"root=/dev/mtdblock3 rootfstype=jffs2\"!\"ubi.mtd=root root=ubi0:rootfs rootfstype=ubifs\"!g /tmp/boot/autoexec*.bat\n"
                if config.plugins.marsu.console.value:
			command +="sed -ie s!\"console=null\"!\"console=ttyS0,115200\"!g /tmp/boot/autoexec*.bat\n"
			command +="sed -ie s!\"quiet\"!\"\"!g /tmp/boot/autoexec*.bat\n"
		else:
			command +="sed -ie s!\"console=ttyS0,115200\"!\"console=null\"!g /tmp/boot/autoexec*.bat\n"
		# make boot filesystem ...
                if config.plugins.marsu.jffs2compression.value == "lzo" or config.plugins.marsu.jffs2compression.value == "none":
			command +="%s/mkfs.jffs2 --root=/tmp/boot --disable-compressor=lzo --compression-mode=none --output=%s/b.jffs2 %s\n" % (marsu_bin,config.plugins.marsu.conversionlocation.value,self.jffs2options)
		else:
			command +="%s/mkfs.jffs2 --root=/tmp/boot --disable-compressor=lzo --compression-mode=size --output=%s/b.jffs2 %s\n" % (marsu_bin,config.plugins.marsu.conversionlocation.value,self.jffs2options)
		if config.plugins.marsu.summary.value is True:  
			command +="%s/sumtool --input=%s/b.jffs2 --output=%s/bs.jffs2 %s\n" % (marsu_bin,config.plugins.marsu.conversionlocation.value,config.plugins.marsu.conversionlocation.value,self.jffs2options)
			command +="cp %s/bs.jffs2 %s/b.jffs2\n" % (config.plugins.marsu.conversionlocation.value,config.plugins.marsu.conversionlocation.value)
			command +="rm %s/bs.jffs2\n" % (config.plugins.marsu.conversionlocation.value)
		# make root filesystem ...
		command +="umount /tmp/boot\n"
		command +="mkdir /tmp/root\n"
		command +="mount -o bind / /tmp/root\n"
                if config.plugins.marsu.ubifsrootcompression.value == "none":
			command +="chattr -R -c /tmp/root\n"
		command +="touch %s/r.ubi\n" % (config.plugins.marsu.conversionlocation.value)
		command +="chmod 777 %s/r.ubi\n" % (config.plugins.marsu.conversionlocation.value)
		if not os.path.exists("/tmp/root/data"):
			command +="mkdir /tmp/root/data\n"
		command +="%s/mkfs.ubifs %s -x %s -r /tmp/root  -o %s/r.ubi\n" % (marsu_bin,self.ubifsrootoptions, config.plugins.marsu.ubifsrootcompression.value, config.plugins.marsu.conversionlocation.value)
		
		if config.plugins.marsu.databackup.value and self.boxtype.startswith("dm7020hd"):  
			# make data filesystem ...
			command +="umount /tmp/data\n"
			command +="mkdir /tmp/data\n"
			command +="mount -o bind /data /tmp/data\n"
       	         	if config.plugins.marsu.ubifsdatacompression.value == "none":
				command +="chattr -R -c /tmp/data\n"
			command +="touch %s/d.ubi\n" % (config.plugins.marsu.conversionlocation.value)
			command +="chmod 777 %s/d.ubi\n" % (config.plugins.marsu.conversionlocation.value)
			command +="%s/mkfs.ubifs %s -x %s -r /tmp/data  -o %s/d.ubi\n" % (marsu_bin,self.ubifsdataoptions, config.plugins.marsu.ubifsdatacompression.value, config.plugins.marsu.conversionlocation.value)
		
		command +="cat /tmp/ubinize.cfg\n"
		command +="%s/ubinize -o %s/r.ubifs %s /tmp/ubinize.cfg\n" % (marsu_bin,config.plugins.marsu.conversionlocation.value,self.ubinizeoptions)
		command +="umount /tmp/root\n"
		command +="rmdir /tmp/root\n"
		command +="chmod 777 %s/b.jffs2\n" % config.plugins.marsu.conversionlocation.value
		command +="chmod 777 %s/b.jffs2\n" % config.plugins.marsu.conversionlocation.value
		command +="chmod 777 %s/r.ubifs\n" % config.plugins.marsu.conversionlocation.value
		if self.boxtype.startswith("dm7020hd"):
			command +="chmod 777 %s/d.ubi\n" % config.plugins.marsu.conversionlocation.value
		command +=self.buildoptions
		command +="rm %s/s.bin\n" % config.plugins.marsu.conversionlocation.value
		command +="rm %s/b.jffs2\n" % config.plugins.marsu.conversionlocation.value
		command +="rm %s/r.ubi\n" % config.plugins.marsu.conversionlocation.value
		command +="rm %s/r.ubifs\n" % config.plugins.marsu.conversionlocation.value
		command +="chmod 777 %s/%s.nfi\n" % (config.plugins.marsu.conversionlocation.value,self.targetname)
		if not self.ownswap:
                	command +="swapoff %s/swapfile\n" % config.plugins.marsu.conversionlocation.value                        
                	command +="rm %s/swapfile\n" % config.plugins.marsu.conversionlocation.value                        
		nfo="%s/%s.nfo" % (config.plugins.marsu.conversionlocation.value,self.targetname)
		command +="echo \"Enigma2: experimental\" > %s\n" % (nfo)
		command +="echo \"Machine: Dreambox %s\" >> %s\n" % (self.boxtype,nfo)
		command +="echo \"Date: %s\" >> %s\n" % (datetime.date.today(),nfo)
                command +="echo \"Issuer: %s\" >> %s\n" % (self.creator,nfo)
	        command +="echo \"Feed: local\" >> %s\n" % nfo
                command +="echo \"Image: local\" >> %s\n" %nfo
		command +="MD5SUM=`md5sum %s/%s.nfi | cut -d\" \" -f 1`\n" % (config.plugins.marsu.conversionlocation.value,self.targetname)
		command +="echo \"MD5: $MD5SUM\" >> %s\n" % nfo
		command +="echo >> %s\n" % nfo
		command +="chmod 777 %s/%s.nfo\n" % (config.plugins.marsu.conversionlocation.value,self.targetname)
		command +="ls -alh %s/%s.*\n" % (config.plugins.marsu.conversionlocation.value,self.targetname)
		command +="rm %s\n" % marsu_busy
		command +="du -h %s/%s.nfi > %s\n" % (config.plugins.marsu.conversionlocation.value,self.targetname,marsu_done)
		command +="df -h\n"
		command +="exit 0\n"
		print command
		b=open(marsu_script,"w")
		b.write(command)
		b.close()
		os_chmod(marsu_script, 0777)

		# here comes my ONE and ONLY container
		self.TimerConversion = eTimer()                                       
		self.TimerConversion.stop()                                           
		self.TimerConversion.timeout.get().append(self.conversionFinishedCheck)
		self.TimerConversion.start(10000,True)                                 
		self.container = eConsoleAppContainer()
              	self.container.execute("%s > %s 2>&1" % (marsu_script,marsu_log))
              	
        def conversionFinishedCheck(self):
		if not os.path.exists(marsu_done):
			# not finished - continue checking ...
 			print "[MARSU] checked if backup is finished ..."
			self.TimerConversion.start(5000,True)                                 
		else:
 			print "[MARSU] found finished backup ..."
			self.TimerConversion = eTimer()                                       
			self.TimerConversion.stop()                                           
			if os.path.exists(marsu_busy):
				os.remove(marsu_busy)
			if os.path.exists(marsu_script):
				os.remove(marsu_script)
			f=open(marsu_done)
			line=f.readline()
			print "[MARSU] found %s" % line
			f.close()
			os.remove(marsu_done)
			sp=[]
			sp=line.split("	")
			print sp
			length=len(sp)
			size=""
			image=""
			path=""
			if length > 0:
				size=sp[0].rstrip().lstrip()
				sp2=[]
				sp2=sp[length-1].split("/")
				print sp2
				length=len(sp2)
				if length > 0:
					image=sp2[length-1]
					path=line.replace(size,"").replace(image,"")
					image=image.replace(".nfi\n","")
					image=image.rstrip().lstrip()
			print "[MARSU] found conversion %s" % line
			self.session.open(MessageBox,size+"B "+_("Image Conversion to %s finished with imagename:\n\n%s.nfi\n\nYou can now flash the created image ...") % (path,image),  MessageBox.TYPE_INFO)                 

class MarsuConfiguration(Screen, ConfigListScreen):
    skin = """
        <screen position="center,80" size="680,300" title="MARSU Configuration" >
	<widget name="logo" position="10,10" size="100,40" transparent="1" alphatest="on" />
        <widget name="config" position="10,60" size="660,240" scrollbarMode="showOnDemand" />
        <widget name="buttonred" position="120,10" size="130,40" backgroundColor="red" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
        <widget name="buttongreen" position="260,10" size="130,40" backgroundColor="green" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
        <widget name="buttonyellow" position="400,10" size="130,40" backgroundColor="yellow" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
        <widget name="buttonblue" position="540,10" size="130,40" backgroundColor="blue" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
        </screen>"""

    def __init__(self, session, args = 0):
	Screen.__init__(self, session)
       	self.list = []
        self.list.append(getConfigListEntry(convertimage_directory, config.plugins.marsu.conversionlocation))
	f=open("/proc/stb/info/model")
        self.boxtype=f.read()
        f.close() 
        self.boxtype=self.boxtype.replace("\n","").replace("\l","")
       	self.list.append(getConfigListEntry(_("Swapsize [256-2047MB]"), config.plugins.marsu.swapsize))
	if self.boxtype =="dm8000":
	       	self.list.append(getConfigListEntry(_("Root Volume Size [60-248MB]"), config.plugins.marsu.volsize))
	elif self.boxtype.startswith("dm7020hd"):
	       	self.list.append(getConfigListEntry(_("Root Volume Size [60-950MB]"), config.plugins.marsu.volsize))
	else:
	       	self.list.append(getConfigListEntry(_("Root Volume Size [40-60MB]"), config.plugins.marsu.volsize))
       	self.list.append(getConfigListEntry(_("Console output"), config.plugins.marsu.console))
	if self.boxtype == "dm7020hd" or self.boxtype =="dm8000":
	       	self.list.append(getConfigListEntry(_("jffs2 erase block summary"), config.plugins.marsu.summary))
     	self.list.append(getConfigListEntry(_("jffs2 boot compression"), config.plugins.marsu.jffs2compression))
     	self.list.append(getConfigListEntry(_("ubifs root compression"), config.plugins.marsu.ubifsrootcompression))
#      	self.list.append(getConfigListEntry(_("ubifs data backup"), config.plugins.marsu.databackup))
	if self.boxtype == "dm7020hd":
	     	self.list.append(getConfigListEntry(_("ubifs data compression"), config.plugins.marsu.ubifsdatacompression))

        self.onShown.append(self.setWindowTitle)
       	ConfigListScreen.__init__(self, self.list)

        # explizit check on every entry
	self.onChangedEntry = []

	self["logo"] = Pixmap()
       	self["buttonred"] = Label(_("Cancel"))
       	self["buttongreen"] = Label(_("OK"))
       	self["buttonyellow"] = Label(disclaimer_string)
	self["buttonblue"] = Label(_("About"))
        self["setupActions"] = ActionMap(["SetupActions", "ColorActions"],
       	{
       		"green": self.save,
        	"red": self.cancel,
	       	"yellow": self.disclaimer,
        	"blue": self.about,
            	"save": self.save,
            	"cancel": self.cancel,
            	"ok": self.save,
       	})
    def setWindowTitle(self):
	self["logo"].instance.setPixmapFromFile("%s/marsu.png" % marsu_plugindir)
	self.setTitle(conversion_string)

    def save(self):
        for x in self["config"].list:
           x[1].save()
        self.close(True)

    def cancel(self):
        for x in self["config"].list:
           x[1].cancel()
        self.close(False)

    def disclaimer(self):
        self.session.open(MessageBox, disclaimer_text, MessageBox.TYPE_WARNING)

    def about(self):
	self.session.open(MessageBox, about_string, MessageBox.TYPE_INFO)

